SystemOrganization addCategory: #DeviceSim!

WAComponent subclass: #WAControlCenter
	instanceVariableNames: ''
	classVariableNames: 'simulation'
	poolDictionaries: ''
	category: 'DeviceSim'!

!WAControlCenter class methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/10/2013 19:30'!
init
	WAAdmin register: WAControlCenter asApplicationAt: 'simulation'.! !

!WAControlCenter class methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/10/2013 19:55'!
reset
	simulation := nil.! !

!WAControlCenter methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/17/2013 10:03'!
initSimulation
	|deviceBuilder|
	
	deviceBuilder := DeviceBuilder defaultBuilder: 'Simulated Device'.
	deviceBuilder ip: '127.0.0.1:8080/devicesim';
	addSensor: (SimulatedSensor newWithType: #TemperatureSensor  name:'Water Heater' min:60  max:80);
	addSensor:  (SimulatedSensor newWithType: #LightSensor name:'Outside Light' min:0  max:1023);
	addSensor:  (OnOffStateSensor newWithType: #LightControl name:'Control Light');
	transport: HttpTransport new.
	
	simulation := Simulation get: 'http://localhost:8080/sensor' deviceBuilder: deviceBuilder.
	simulation transport: (deviceBuilder transport).
	simulation start.! !

!WAControlCenter methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/10/2013 19:36'!
initialize
	|deviceBuilder device|
	super initialize.
	simulation isNil ifTrue:[
		self initSimulation.
	].! !

!WAControlCenter methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/10/2013 23:46'!
renderContentOn: html
	html div class:'heading'; with: [
		html text:'Device Simulation control center'
		].
	
	html div class:'btn'; with:  [
	html anchor
		callback: [
				Transcript show: 'Starting...'.
				simulation stop.
				self initSimulation.

				self inform: 'Simulation started successfully.'.
			 ];
		with: 'Start'.
	].
	
	html div class:'btn'; with:  [
	html anchor
		callback: [
				Transcript show: 'Stopping...'.
				simulation stop.
				self inform: 'Simulation stopped successfully.'.
			 ];
		with: 'Stop'.
	].! !

WARestfulHandler subclass: #DeviceInfoWS
	instanceVariableNames: 'device'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DeviceSim'!

!DeviceInfoWS methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/5/2013 14:37'!
device: aDevice
	device := aDevice.! !

!DeviceInfoWS methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/5/2013 14:39'!
getDeviceInfo
    <get>
    <path: 'info'>
    <produces: 'text/xml'>
	
	Transcript show: device.
	
	^device asXml.! !

!DeviceInfoWS methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/19/2013 02:03'!
sendCommand: key value: value
    <post>
    <path: 'command/{1}/{2}'>
    <produces: 'text/xml'>
	Transcript show: device.
	^'<success />'! !

TestCase subclass: #DeviceBuilderTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DeviceSim'!

!DeviceBuilderTest methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/20/2013 15:26'!
testSimpleBuilder
	|deviceBuilder device simulation|
	deviceBuilder := DeviceBuilder defaultBuilder: 'Simulated Device'.
	deviceBuilder addSensor: (SimulatedSensor newWithType: #TemperatureSensor  min:60  max:80);
	addSensor:  (SimulatedSensor newWithType: #LightSensor min:0  max:1023);
	
	addSensor:  (OnOffStateSensor newWithType: #LightControl);
	
	transport: HttpTransport new.
	
	simulation := Simulation get: 'http://localhost:8080/sensor' deviceBuilder: deviceBuilder.
	simulation transport: (deviceBuilder transport).
	simulation start.
	! !

TestCase subclass: #SimulatedSensorTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DeviceSim'!

!SimulatedSensorTest methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/10/2013 23:52'!
testRandomValueRange
	|sensor value|
	sensor := SimulatedSensor newWithType: #LightSensor name:'sensor' min: 0 max: 1024.
	value := sensor value.
	self assert: value>=0.
	self assert: value<=1024.
	
	sensor := SimulatedSensor newWithType: #TemperatureSensor name:'sensor 2' min: 60 max: 100.
	value := sensor value.
	self assert: value>=60.
	self assert: value<=100.
	
	sensor := OnOffStateSensor newWithType: #LightControl.
	value := sensor value.! !

DeviceSensor subclass: #SimulatedSensor
	instanceVariableNames: 'max min'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DeviceSim'!

SimulatedSensor subclass: #OnOffStateSensor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DeviceSim'!

!OnOffStateSensor class methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/20/2013 15:26'!
newWithType: aType name:aName
	^OnOffStateSensor new type: aType name: aName.
	

	! !

!OnOffStateSensor methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/10/2013 18:38'!
initialize
	super initialize.
	self min: 0.
	self max: 100.! !

!OnOffStateSensor methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/17/2013 09:27'!
randomValue
	 (super randomValue asInteger rem: 2) == 0 ifTrue: [^0] ifFalse: [^1].! !

!OnOffStateSensor methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/17/2013 09:25'!
type: aType name: aName
	type := aType.
	name := aName.! !

!OnOffStateSensor methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/10/2013 18:24'!
value
	^self randomValue! !

!SimulatedSensor class methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/17/2013 09:15'!
newWithType: aType name: aName
	^SimulatedSensor new type: aType  name: aName.! !

!SimulatedSensor class methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/10/2013 23:49'!
newWithType: aType name: aName min: aMin max: aMax
	^SimulatedSensor new type: aType  name: aName min: aMin max: aMax.! !

!SimulatedSensor methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/10/2013 18:21'!
initialize
	^super initialize.! !

!SimulatedSensor methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/5/2013 14:14'!
max
	^max! !

!SimulatedSensor methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/5/2013 14:14'!
max: aMax
	max := aMax! !

!SimulatedSensor methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/5/2013 14:14'!
min
	^min! !

!SimulatedSensor methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/5/2013 14:14'!
min: aMin
	min := aMin! !

!SimulatedSensor methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/5/2013 14:16'!
randomValue
	 ^Random new next * (max - min) + min.! !

!SimulatedSensor methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/11/2013 21:55'!
setDefaultName
	name := #DummyName! !

!SimulatedSensor methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/10/2013 18:22'!
setType
	type := #SimulatedSensor! !

!SimulatedSensor methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/10/2013 18:22'!
setUnit
	dataUnit := ''! !

!SimulatedSensor methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/10/2013 17:50'!
type: aType min: aMin max: aMax
	type := aType.
	min := aMin.
	max := aMax.! !

!SimulatedSensor methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/17/2013 09:16'!
type: aType  name: aName
	type := aType.
	name:=aName.
	min:=0.
	max:=100.! !

!SimulatedSensor methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/10/2013 23:50'!
type: aType  name: aName min: aMin max: aMax
	type := aType.
	name:=aName.
	min := aMin.
	max := aMax.
	! !

!SimulatedSensor methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/5/2013 14:17'!
unit
	self subclassResponsibility! !

!SimulatedSensor methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/10/2013 17:48'!
value
	^self randomValue! !

Object subclass: #DataPusher
	instanceVariableNames: 'device transport scheduler task serverUrl'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DeviceSim'!

!DataPusher class methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/11/2013 00:06'!
newWithDevice: adevice transport: atransport serverUrl: aUrl
	^DataPusher new device: adevice transport: atransport serverUrl: aUrl.! !

!DataPusher methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/11/2013 00:06'!
device: aDevice transport: aTransport serverUrl: aUrl
	device := aDevice.
	transport := aTransport.
	serverUrl := aUrl.! !

!DataPusher methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/10/2013 23:36'!
initialize
	super initialize.
	scheduler :=TaskScheduler new.! !

!DataPusher methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/17/2013 09:43'!
pushData
	|url|
	Transcript show: 'push data'.
	device sensors valuesDo: [:sensor | 
		
	 url := serverUrl , '/Data/', (device name), '/' , (sensor name).
       transport sendData: url payload:  (sensor asDataXml).
	
	].
! !

!DataPusher methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/10/2013 23:47'!
start
	Transcript show: 'starting'.
	scheduler start.
	task := scheduler do: [ self pushData]
   	every: 10 seconds.
! !

!DataPusher methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/10/2013 23:47'!
stop
	Transcript show: scheduler.
	task isNil ifFalse: [
	scheduler removeTask: task.
	].
	task := nil.
	scheduler stop.
	! !

Object subclass: #DeviceBuilder
	instanceVariableNames: 'name sensors transport interval ip'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DeviceSim'!

!DeviceBuilder class methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/10/2013 17:43'!
defaultBuilder: aName
	^DeviceBuilder new name: aName.! !

!DeviceBuilder methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/10/2013 19:42'!
addSensor: aSensor
	sensors at: (aSensor type) put:aSensor.! !

!DeviceBuilder methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/17/2013 09:48'!
build
	|device|
	device := Device new.
	device name: name.
	device ip: ip.
	sensors valuesDo: [:sensor | device addSensor: sensor].
	
	
	"start the transport"
	transport device: device.
	^device! !

!DeviceBuilder methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/5/2013 10:43'!
initialize
	super initialize.
	sensors := Dictionary new.
	! !

!DeviceBuilder methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/17/2013 09:48'!
ip: anIp
	ip := anIp.! !

!DeviceBuilder methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/10/2013 17:24'!
name:aName
	name := aName.! !

!DeviceBuilder methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/5/2013 14:51'!
sampleInterval: aInterval
	interval := aInterval! !

!DeviceBuilder methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/5/2013 15:09'!
transport
	^transport! !

!DeviceBuilder methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/5/2013 15:11'!
transport: aTransport
	transport := aTransport.! !

Object subclass: #SimulatedTransport
	instanceVariableNames: 'device serverUrl'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DeviceSim'!

SimulatedTransport subclass: #HttpTransport
	instanceVariableNames: 'deviceInfoWS'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DeviceSim'!

!HttpTransport methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/6/2013 10:23'!
sendData: path payload: payload
	|response|
	
	response := ZnClient new 
    	post: path contents: payload;
    	response.
	! !

!HttpTransport methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/10/2013 17:42'!
start
	|response url|
	"start http server"
	deviceInfoWS := WAAdmin register: DeviceInfoWS at: 'devicesim'.
	deviceInfoWS device: device.
	.
	"send initial discovery message"
	url := serverUrl , '/Device'.
	self sendData: url payload: (device asXml).
	! !

!SimulatedTransport methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/5/2013 14:45'!
device: aDevice
	device := aDevice.! !

!SimulatedTransport methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/6/2013 10:23'!
sendData: path payload: payload
	self subclassResponsibility.! !

!SimulatedTransport methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/5/2013 15:03'!
serverUrl: aServerAddress
	serverUrl := aServerAddress.! !

!SimulatedTransport methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/5/2013 14:25'!
start
	self subclassResponsibility ! !

Object subclass: #Simulation
	instanceVariableNames: 'deviceBuilder serverAddress transport dataPusher datapusher'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DeviceSim'!

!Simulation class methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/5/2013 14:55'!
get: serverAddress deviceBuilder: aDeviceBuilder
	^Simulation new serverAddress: serverAddress deviceBuilder: aDeviceBuilder.! !

!Simulation methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/5/2013 14:27'!
deviceBuilder: aDeviceBuilder
	deviceBuilder := aDeviceBuilder.! !

!Simulation methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/5/2013 14:56'!
serverAddress: aServerAddress deviceBuilder: aDeviceBuilder.
	serverAddress := aServerAddress.
	deviceBuilder := aDeviceBuilder.! !

!Simulation methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/11/2013 00:06'!
start
	|device|
	device := deviceBuilder build.
	
	transport serverUrl: serverAddress.
	transport start.
	
	"initialize period updates"
	dataPusher := DataPusher newWithDevice: device transport: transport serverUrl: serverAddress.
	dataPusher start.! !

!Simulation methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/10/2013 19:25'!
stop
	dataPusher stop.! !

!Simulation methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/5/2013 15:10'!
transport: aTransport
	transport := aTransport.! !
